#include <glib.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "tokens.h"

void fatal(char *errmsg)
{
	fprintf(stderr, errmsg);
	exit(1);
}

/* returns a GHashTable of word frequencies in inF */
GHashTable *getWordFreq(FILE *inF)
{
  char *word;
  int *count;
  GHashTable *whash=g_hash_table_new(g_str_hash, g_str_equal);
  
  while ((word=getNextWord(inF))) {
    if ((count = g_hash_table_lookup(whash, word))) {
      (*count)++;
    } else {
			char *newWord = strdup(word);
      count = malloc(sizeof(int));
      if (!count || !newWord) {
        fatal("Out of memory");
      }
      *count = 1;
      g_hash_table_insert(whash, newWord, count);
    }
  }
  return whash;
}


/* to be called from a g_hash_table_foreach -- prints a key/value pair */
static void printpair(gpointer key, gpointer value, gpointer user_data)
{
  printf("%s: %d\n", (char*)key, *(int*)value);
}


/* prints the word frequency information contained in wordfreq */
void printWordFreqs(GHashTable *wordfreq)
{
  g_hash_table_foreach(wordfreq, printpair, NULL);
}


int main(void)
{
  GHashTable *wordfreq;

  wordfreq = getWordFreq(stdin);
  printWordFreqs(wordfreq);
  return 0;
}
